# Script 1. First Script to aggregate CDL classes. Requires each year of CDL to be recoded as describe in, "APPENDIX
# 1-6: CDL to UDL  Crosswalk (DOCX)", posted with the Listed Species Biological Evaluations:

# https://www.epa.gov/endangered-species/implementing-nas-report-recommendations-ecological-risk-assessment-endangered-and

# This can be done using ArcToolbox Spatial Analyst Tools/Reclass/Reclass by Table, using "cdl_recode_new.txt" loaded
# as a remap table. These scripts demonstrate 5 years of CDL (2013-2017); the previously released draft was 6 years
# (2010-2015). Edit scripts as needed for time frame desired.

# All paths are relative and need updating. Suggested directory structure needs manual creation in most cases.

# This script has been approved for release by the U.S. Environmental Protection Agency (USEPA). Although
# the script has been subjected to rigorous review, the USEPA reserves the right to update the script as needed
# pursuant to further analysis and review. No warranty, expressed or implied, is made by the USEPA or the U.S.
# Government as to the functionality of the script and related material nor shall the fact of release constitute
# any such warranty. Furthermore, the script is released on condition that neither the USEPA nor the U.S. Government
# shall be held liable for any damages resulting from its authorized or unauthorized use.

__author__ = 'Steven Lennartz'

# Import Site Packages
import arcpy
from arcpy import env
from arcpy.sa import *
import datetime

# Check out the ArcGIS Spatial Analyst extension license
arcpy.CheckOutExtension("Spatial")

# Set variables. All paths are relative and need updating.
# Create a File Geodatabase, populated with recoded CDL layers as described in the header. This will be your in-
# workspace.
inws = r".../CDL/CDL_Recodes/180524/CDL_Reclass_1317_180524.gdb/"
# Create a File Geodatabase for an out-workspace
outws = r".../CDL/CDL_Recodes/180524/CDL_GenAggregates_1317_180524.gdb/"
# The cultivated layer from the CDL website
cult = Raster(".../CDL/Cultivated_Layer/2017_Cultivated_Layer/2017_Cultivated_Layer.img")

# Environmental settings
arcpy.env.workspace = inws
arcpy.env.overwriteOutput = 1
arcpy.env.snapRaster = cult
arcpy.env.extent = cult
arcpy.env.scratchWorkspace = outws
arcpy.env.parallelProcessingFactor = "100%"

# Create a list of the rasters in the in-workspace
ras = arcpy.ListRasters()

# The Key is included as a txt (add file extension if using txt format)
key = "EPA_CDL_Recode_Key_180424"
valueList = {row[0] for row in arcpy.da.SearchCursor(key, "Value")}

start_t1 = datetime.datetime.now()

# Aggregate multiple years of CDL
# For loop
for value in valueList:
    if arcpy.Exists("{}CDL_1317_{}".format(outws, value)):
        print "{} Exists".format("CDL_1317_{}".format(value))
    else:
        start_t = datetime.datetime.now()
        print "Temporally Aggregating Class {} at {}".format(value, start_t)
        # Conditional statement aggregating 5 years of CDL
        out = Con(((Raster(ras[0]) == value) | (Raster(ras[1]) == value) | (Raster(ras[2]) == value) | (Raster(ras[
            3]) == value) | (Raster(ras[4]) == value)), 1, 0)
        out.save("{}CDL_1317_{}".format(outws, value))
        print "Building Pyramids....."
        arcpy.BuildPyramids_management("{}CDL_1317_{}".format(outws, value))
        print "Completed Temporal Aggregation of Class {} in: {}".format(value, datetime.datetime.now() - start_t)

print "Completed Temporal Aggregation of all Classes in: {}".format(datetime.datetime.now() - start_t1)
