# Script 3. This script clips each raster from script 2 by county. These will serve as an input for the expansion
# script.

# This script has been approved for release by the U.S. Environmental Protection Agency (USEPA). Although
# the script has been subjected to rigorous review, the USEPA reserves the right to update the script as needed
# pursuant to further analysis and review. No warranty, expressed or implied, is made by the USEPA or the U.S.
# Government as to the functionality of the script and related material nor shall the fact of release constitute
# any such warranty. Furthermore, the script is released on condition that neither the USEPA nor the U.S. Government
# shall be held liable for any damages resulting from its authorized or unauthorized use.

__author__ = 'Steven Lennartz'

# Import Site Packages
import arcpy
import datetime
from arcpy import env
from arcpy.sa import *

# Check out the ArcGIS Spatial Analyst extension license
arcpy.CheckOutExtension("Spatial")

# Set variables
# In-workspace is the out-workspace from step 2
inws = r".../CDL/CDL_Recodes/180524/CDL_GenClass13_1317_180524.gdb/"
# Output directory for
outdir = r".../CDL/Cnty_Clips/180524/1317/"
# Tiger Line County boundaries, vector
cnty = r".../CDL/Boundaries/St_Cnty_Boundaries.gdb/tl_2013_cnty_conus_alb"
# Unique ID for counties, "FIPS code"
fipscode = "GEOID"
# The cultivated layer from the CDL website
cult = Raster(".../CDL/Cultivated_Layer/2017_Cultivated_Layer/2017_Cultivated_Layer.img")

start_t = datetime.datetime.now()

# Environmental Settings
arcpy.env.overwriteOutput = 1
arcpy.env.workspace = inws
arcpy.env.snapRaster = cult
arcpy.env.parallelProcessingFactor = "100%"
print "Starting process at: {}".format(start_t)

# List county FIPS codes
uniqueFips = {(row[0], row[1]) for row in arcpy.da.SearchCursor(cnty, (fipscode, "SHAPE@"))}

# Create list of rasters in current workspace
rasList = arcpy.ListRasters()

# Clip each raster by county FIPS code
# Environmental setting, extent
arcpy.env.extent = "MAXOF"
clipfc = "clip_lyr"
# Make county feature layer
arcpy.MakeFeatureLayer_management(cnty, clipfc)

# Loop through raster list
for raster in rasList:
    start_t2 = datetime.datetime.now()
    # return string crop code from raster name
    arcpy.env.workspace = inws
    desc = arcpy.Describe(raster)
    name = desc.name
    val = name.split('_')[2]

    # check for existing geodatabase
    outws = r"{}CDL_{}_cnty.gdb".format(outdir, val)
    if not arcpy.Exists(outws):
        arcpy.CreateFileGDB_management(outdir, "CDL_{}_cnty.gdb".format(val))
    arcpy.env.workspace = outws

    # Loop through counties to clip raster layers
    for fips, shape in uniqueFips:
        start_t1 = datetime.datetime.now()
        extent = shape.extent
        extent = "{} {} {} {}".format(extent.XMin, extent.YMin, extent.XMax, extent.YMax)
        arcpy.SelectLayerByAttribute_management(clipfc, "NEW_SELECTION", "{} = '{}'".format(fipscode, fips))
        in_file = "{}{}".format(inws, raster)
        out_file = "{}/CDL_{}_{}".format(outws, val, fips)
        if not arcpy.Exists(out_file):
            print "Clipping CDL_{}_{}...".format(val, fips)
            arcpy.Clip_management(in_file, extent, out_file, clipfc, "2147483647", "ClippingGeometry", "NO_MAINTAIN_EXTENT")
            print "Finished clip in: {}".format(datetime.datetime.now() - start_t1)
    print "Completed Clipping {} in: {}".format(val, datetime.datetime.now() - start_t2)
print "Completed All County Clipping in: {}".format(datetime.datetime.now() - start_t)
