# Script 4. This script updates CDL acreage values for the table that compares CDL and Census county acreages. It
# requires a geodatabase populated with tables generated using 'Spatial Analyst/Zonal/Tabulate Area' using each layer
# generated in script 2 and the Tiger county boundaries.

# This script has been approved for release by the U.S. Environmental Protection Agency (USEPA). Although
# the script has been subjected to rigorous review, the USEPA reserves the right to update the script as needed
# pursuant to further analysis and review. No warranty, expressed or implied, is made by the USEPA or the U.S.
# Government as to the functionality of the script and related material nor shall the fact of release constitute
# any such warranty. Furthermore, the script is released on condition that neither the USEPA nor the U.S. Government
# shall be held liable for any damages resulting from its authorized or unauthorized use.

__author__ = 'Steven Lennartz'

# Import Site Packages
import arcpy
import datetime
from arcpy import env
from arcpy.sa import *

# Set variables
# In-workspace
inws = r".../CDL/Analysis/NASS2012_CDL1317_180524.gdb/"
# Master table. Table header should have FIPS code, 13 CDL columns, and 13 Census columns. POC for this table is
# Kurt Pluntke. Table will be updated in the future to include date generated using instructions in ATTACHMENT 1-3;
# Explanation of Census Of Agriculture Table Generation
intbl = "NASS2012_CDL1317_180524"

# Environmental Settings
arcpy.env.workspace = inws
arcpy.CheckOutExtension("Spatial")
arcpy.env.overwriteOutput = 1
arcpy.env.parallelProcessingFactor = "100%"

start_t1 = datetime.datetime.now()

# Create a table view from master table
print "Converting {} to Table View at: {}".format(intbl, start_t1)
tblView = "TableView"
arcpy.MakeTableView_management(intbl, tblView)

# List tables generated from "tabulate area" command, described at the top of this script.
tblList = arcpy.ListTables("*area")

# Loop individual area tables to update master table
for tbl in tblList:
    start_t = datetime.datetime.now()
    print "Processing table {} at: {}".format(tbl, start_t)
    # return string crop code from tbl name
    desc = arcpy.Describe(tbl)
    name = desc.name
    val = name.split('_')[2]

    # add join intbl with tbl (FIPS, GEOID)
    arcpy.AddJoin_management(tblView, "FIPS", tbl, "GEOID", "KEEP_COMMON")

    # calculate corresponding CDL field by converting VALUE_1 from square meters to acres (1 sq meter is 0.000247 acres)
    outval = "{}.CDL_{}".format(intbl, val)
    inval = "CDL_1317_{}_180524_cnty_area.VALUE_1".format(val)
    arcpy.CalculateField_management(tblView, outval, "(!{}! * 0.000247)".format(inval), "PYTHON")
    arcpy.RemoveJoin_management(tblView)
    print "Completed {} table update in: ".format(datetime.datetime.now() - start_t)

print "Completed updating all tables in: {}".format(datetime.datetime.now() - start_t1)
