# Script 7. Mosaic the county based expansions from script 6.

# This script has been approved for release by the U.S. Environmental Protection Agency (USEPA). Although
# the script has been subjected to rigorous review, the USEPA reserves the right to update the script as needed
# pursuant to further analysis and review. No warranty, expressed or implied, is made by the USEPA or the U.S.
# Government as to the functionality of the script and related material nor shall the fact of release constitute
# any such warranty. Furthermore, the script is released on condition that neither the USEPA nor the U.S. Government
# shall be held liable for any damages resulting from its authorized or unauthorized use.

__author__ = 'Steven Lennartz'

# Import Site Packages
import arcpy
import datetime
from arcpy import env
from arcpy.sa import *

# Set Variables
# Key table for general class crop names and codes
classcode = r".../CDL/Analysis/NASS2012_CDL1317_180524.gdb/NASS2012_Genclass13_codes"
# Crop code field name. These codes are arbitrary, increments of 10, from 10 through 110.
classfield = "GenClassID"
# Master table from script 4
acres = r".../CDL/Analysis/NASS2012_CDL1317_180524.gdb/NASS2012_CDL1317_180524"
# Unique ID for counties, "FIPS code"
geoid = "FIPS"

# In-workspace is the output directory from script 3
inws = r".../CDL/Cnty_Clips/180524/1317/"
# Output directory
outws = r".../CDL/Expand/180524/1317/Mosaic/"

start_t = datetime.datetime.now()

# Environmental settings
arcpy.env.overwriteOutput = 1
arcpy.env.snapRaster = Raster(".../CDL/Cultivated_Layer/2017_Cultivated_Layer/2017_Cultivated_Layer.img")
arcpy.env.parallelProcessingFactor = "100%"

print "Begun processing at: {}".format(start_t)

# Check out the ArcGIS Spatial Analyst extension license
arcpy.CheckOutExtension("Spatial")

# Get list of class codes
uniqueCrops = {row[0] for row in arcpy.da.SearchCursor(classcode, classfield)}

# Get FIPS codes
uniqueFips = {row[0] for row in arcpy.da.SearchCursor(acres, geoid)}

# check for existing geodatabase
if not arcpy.Exists("{}CDL1317x_mosaic.gdb".format(outws)):
    print "Creating CDL1317x_mosaic.gdb"
    arcpy.CreateFileGDB_management(outws, "CDL1317x_mosaic.gdb")

# Set output geodatabase
outgdb = "{}CDL1317x_mosaic.gdb".format(outws)

# Mosaic loop by crop
for crop in uniqueCrops:
    crop = int(crop)
    crop = str(crop)

    croptime1 = datetime.datetime.now()
    print "starting {} mosaic at: {}".format(crop, croptime1)

    # Expanded geodatabase from script 6
    xgdb = ".../CDL/Expand/180524/1317/CDL{}x_cnty.gdb".format(crop)
    arcpy.env.workspace = xgdb
    # Create Raster Catalog
    cat = "{}/CDL{}x_cat".format(outgdb, crop)
    if arcpy.Exists(cat):
        print "{} Exists".format(cat)
    if not arcpy.Exists(cat):
        print "Creating Raster Catalog CDL{}x_cat".format(crop)
        arcpy.CreateRasterCatalog_management(outgdb, "CDL{}x_cat".format(crop))
        print "Copying Expanded Rasters to Catalog CDL{}x_cat".format(crop)
        arcpy.WorkspaceToRasterCatalog_management(xgdb, cat)

 # Load non-expanded rasters into Raster Catalog
    print "Loading non-expanded rasters into CDL{}x_cat".format(crop)
    for fips in uniqueFips:
        timex2 = datetime.datetime.now()
        cdlx = "{}/CDL{}x_{}".format(xgdb, crop, fips)
        if arcpy.Exists(cdlx):
            print "CDL{}x_{} exists".format(crop, fips)
        if not arcpy.Exists(cdlx):
            if arcpy.Exists("{}/CDL_{}_{}".format(cat, crop, fips)):
                print "{}/CDL_{}_{} Exists".format(cat, crop, fips)
            try:
                if not arcpy.Exists("{}/CDL_{}_{}".format(cat, crop, fips)):
                    arcpy.CopyRaster_management("{0}CDL_{1}_cnty.gdb/CDL_{1}_{2}".format(inws, crop, fips), "{}/CDL_{}_{}".format(cat, crop, fips))
                    print "Added CDL_{}_{} to Catalog in {}".format(crop, fips, datetime.datetime.now() - timex2)
            except ValueError, e:
                print e
                continue

    # Mosaic Raster Catalog to new dataset
    print "Raster CDL{}x_cat to Raster Dataset at: {}".format(crop, datetime.datetime.now())
    arcpy.RasterCatalogToRasterDataset_management(cat, "{}/CDL_1317_{}x".format(outgdb, crop))
    print "......Building pyramids"
    arcpy.BuildPyramids_management("{}/CDL_1317_{}x".format(outgdb, crop))
    print "......Building attribute table"
    arcpy.BuildRasterAttributeTable_management("{}/CDL_1317_{}x".format(outgdb, crop), "NONE")
    print "..CDL_1317_{}x mosaic complete in {}".format(crop, (datetime.datetime.now()-croptime1))

print "completed mosaic in: {}".format(datetime.datetime.now() - start_t)
