# Script 8. Some county FIPS codes from Tiger County Boundaries are not reported in the Ag Census. Those counties do
# not join in the expansion script 6 and result in NoData values. This script corrects for NoData.

# This script has been approved for release by the U.S. Environmental Protection Agency (USEPA). Although
# the script has been subjected to rigorous review, the USEPA reserves the right to update the script as needed
# pursuant to further analysis and review. No warranty, expressed or implied, is made by the USEPA or the U.S.
# Government as to the functionality of the script and related material nor shall the fact of release constitute
# any such warranty. Furthermore, the script is released on condition that neither the USEPA nor the U.S. Government
# shall be held liable for any damages resulting from its authorized or unauthorized use.

__author__ = ' Steven Lennartz'

# Import Site Packages
import arcpy
import datetime
from arcpy import env
from arcpy.sa import *

# Set Variables
# In-workspace is the mosaic geodatabase from script 7
inws = r".../CDL/Expand/180524/1317/Mosaic/CDL1317x_mosaic.gdb"
# Out-workspace from script 2 supplies replacement values for NoData
patch = r".../CDL/CDL_Recodes/180524/CDL_GenClass13_1317_180524.gdb/"

# Environmental settings
arcpy.env.workspace = inws
arcpy.env.overwriteOutput = 1
arcpy.env.snapRaster = Raster(".../CDL/Cultivated_Layer/2017_Cultivated_Layer/2017_Cultivated_Layer.img")
arcpy.env.parallelProcessingFactor = "100%"
arcpy.env.scratchWorkspace = patch

# Check out the ArcGIS Spatial Analyst extension license
arcpy.CheckOutExtension("Spatial")

start_t = datetime.datetime.now()
print "Begun processing at: {}".format(start_t)

# check for existing geodatabase
if not arcpy.Exists(".../CDL/Expand/180524/1317/Mosaic/Final/CDL1317x_mosaic2.gdb"):
    print "Creating CDL1317x_mosaic2.gdb"
    arcpy.CreateFileGDB_management(".../CDL/Expand/180524/1317/Mosaic/Final/", "CDL1317x_mosaic2.gdb")
outws = r".../CDL/Expand/180524/1317/Mosaic/Final/CDL1317x_mosaic2.gdb/"

# List rasters in workspace
rasterList = arcpy.ListRasters()
# Loop through rasters to correct NoData
for raster in rasterList:
    start_t1 = datetime.datetime.now()
    print "Processing {} at {}".format(raster, datetime.datetime.now())
    inras = Raster(raster)
    desc = arcpy.Describe(inras)
    name = desc.name
    val = name.split('_')[2]
    val = val[:-1]
    true = Raster("{}CDL_1317_{}".format(patch, val))
    out = Con(IsNull(inras), true, inras)
    out.save("{}CDL_1317_{}x2".format(outws, val))
    print "Building Pyramids....."
    arcpy.BuildPyramids_management("{}CDL_1317_{}x2".format(outws, val))
    print "Completed NoData holes for Class {} in: {}".format(val, datetime.datetime.now() - start_t1)
print "Completed NoData holes for all classes in: {}".format(datetime.datetime.now() - start_t)
